@@ -24,6 +24,13 @@
 #include "DesktopJob.h"
 #include "FrameJob.h"
 
+#include <gio/gio.h>
+#include <libdbusmenu-glib/server.h>
+
+#include <rtl/process.h>
+#include <osl/diagnose.h>
+#include <cppuhelper/implbase1.hxx>
+
 #include <com/sun/star/beans/XPropertySet.hpp>
 #include <com/sun/star/document/XEventBroadcaster.hpp>
 #include <com/sun/star/document/XEventListener.hpp>
@@ -37,16 +44,6 @@
 #include <com/sun/star/frame/XModel.hpp>
 #include <com/sun/star/frame/XLayoutManager.hpp>
 
-
-#include <gio/gio.h>
-#include <libdbusmenu-glib/server.h>
-
-#include <rtl/process.h>
-#include <osl/diagnose.h>
-#include <cppuhelper/implbase1.hxx>
-
-
-
 using rtl::OUString;
 using com::sun::star::beans::XPropertySet;
 using com::sun::star::beans::NamedValue;
@@ -71,34 +68,7 @@ using com::sun::star::uno::UNO_QUERY;
 using com::sun::star::uno::UNO_QUERY_THROW;
 using com::sun::star::uno::XInterface;
 
-//Attempt to get the welcome window frame to hide the menubar
-/*class DesktopFrameHelper : public cppu::WeakImplHelper1 < XEventListener >
-{
-  public:
-	DesktopFrameHelper()
-	{
-	}
-
-	virtual ~DesktopFrameHelper()
-	{
-	}
-	
-	virtual void SAL_CALL notifyEvent(const com::sun::star::document::EventObject& action)
-	  throw (RuntimeException)
-	{
-		rtl::OString name = rtl::OUStringToOString (action.EventName, RTL_TEXTENCODING_ASCII_US);
-		g_debug ("frame action 00-- %s", name.getStr());
-	}
-	
-	virtual void SAL_CALL disposing(const EventObject& aEvent)
-	  throw (RuntimeException)
-	{
-		g_debug ("frame disposed");
-	}
-};*/
-
-
-//--- DBus Callbacks ---//
+//-------------------------- D-Bus Callbacks ----------------------------------
 static void
 on_bus (GDBusConnection * connection,
         const gchar * name,
@@ -115,7 +85,7 @@ name_lost (GDBusConnection * connection, const gchar * name, gpointer user_data)
 	return;
 }
 
-
+// --------------------------- DesktopJob ----------------------------------
 Any SAL_CALL DesktopJob::execute( const Sequence< NamedValue >& aArguments )
 	throw ( IllegalArgumentException, Exception, RuntimeException )
 {
@@ -129,13 +99,7 @@ Any SAL_CALL DesktopJob::execute( const Sequence< NamedValue >& aArguments )
                name_lost,
                NULL,
                NULL);
-	
-	//Intent to export the welcome window
-	//TODO: Get the Window events right for this, try to not use Jobs.cxu for FrameJob
-	//Reference < XEventBroadcaster > xEB(mxMSF->createInstance(OUString ( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.frame.GlobalEventBroadcaster"))),
-	//                                    UNO_QUERY);
-	//xEB->addEventListener (new DesktopFrameHelper ());
-	
+
 	return Any ();
 }
 
