@@ -23,6 +23,8 @@
 
 #include "FrameHelper.h"
 #include "AwtKeyToDbusmenuString.h"
+#include "MenuItemInfo.h"
+#include "MenuItemStatusListener.h"
 
 #include <com/sun/star/awt/KeyEvent.hpp>
 #include <com/sun/star/awt/SystemDependentXWindow.hpp>
@@ -30,6 +32,7 @@
 #include <com/sun/star/awt/XWindow2.hpp>
 #include <com/sun/star/awt/Key.hpp>
 #include <com/sun/star/awt/KeyModifier.hpp>
+#include <com/sun/star/awt/MenuEvent.hpp>
 #include <com/sun/star/beans/XPropertySet.hpp>
 #include <com/sun/star/container/XNameAccess.hpp>
 #include <com/sun/star/awt/XPopupMenu.hpp>
@@ -47,7 +50,6 @@
 #include <com/sun/star/frame/XPopupMenuController.hpp>
 #include <com/sun/star/frame/FrameAction.hpp>
 #include <com/sun/star/frame/FrameActionEvent.hpp>
-#include <com/sun/star/frame/status/Visibility.hpp>
 #include <com/sun/star/lang/SystemDependent.hpp>
 #include <com/sun/star/beans/PropertyValue.hpp>
 #include <com/sun/star/util/URL.hpp>
@@ -70,7 +72,9 @@ using rtl::OUStringToOString;
 using namespace ::com::sun::star;
 
 using com::sun::star::awt::KeyEvent;
+using com::sun::star::awt::MenuEvent;
 using com::sun::star::awt::SystemDependentXWindow;
+using com::sun::star::awt::XMenuListener;
 using com::sun::star::awt::XMenuExtended;
 using com::sun::star::awt::XMenuListener;
 using com::sun::star::awt::MenuEvent;
@@ -91,13 +95,12 @@ using com::sun::star::frame::XModel;
 using com::sun::star::frame::XModuleManager;
 using com::sun::star::frame::XLayoutManager;
 using com::sun::star::frame::XPopupMenuController;
-using com::sun::star::frame::FeatureStateEvent;
-using com::sun::star::frame::status::Visibility;
 using com::sun::star::lang::SystemDependent::SYSTEM_XWINDOW;
 using com::sun::star::uno::UNO_QUERY;
 using com::sun::star::uno::UNO_QUERY_THROW;
 using com::sun::star::uno::Sequence;
 using com::sun::star::uno::XComponentContext;
+using com::sun::star::uno::XInterface;
 using com::sun::star::ui::XUIElement;
 using com::sun::star::ui::XUIConfigurationManager;
 using com::sun::star::ui::XUIConfigurationManagerSupplier;
@@ -106,9 +109,7 @@ using com::sun::star::ui::XModuleUIConfigurationManagerSupplier;
 using com::sun::star::util::URL;
 using com::sun::star::util::XURLTransformer;
 
-// Item callbacks
-OString  xid_to_object_path (unsigned long xid);
-
+// ------------------------ Item callbacks ---------------------------
 // Item activated. It distpatches the command associated to a given menu item.
 void
 item_activated (DbusmenuMenuitem *item, guint timestamp, gpointer user_data)
@@ -132,8 +133,17 @@ item_about_to_show (DbusmenuMenuitem *item, gpointer user_data)
 	                                 UNO_QUERY);
 	Reference < XPropertySet > menuPropSet (menuBar, UNO_QUERY);
 	
+	if (!menuPropSet.is ())
+	{
+		return FALSE;
+	}
+	
 	Reference < XMenu > xMenu(menuPropSet->getPropertyValue(OUString::createFromAscii("XMenuBar")),
 	                          UNO_QUERY);
+	if (!xMenu.is())
+	{
+		return FALSE;
+	}
 	
 	//Find xMenu for the first level item
 	Reference < XMenu > xSubMenu;
@@ -174,191 +184,13 @@ destroy_menuitem (gpointer data)
 	g_object_unref (G_OBJECT (data));
 }
 
-// This is used in a hash table with commandurls as keys
-class MenuItemInfo {
-	gchar*            label;         //Label text in UTF-8 with tildes subst by underscores
-	gint              check_state;
-	gchar*            check_type;
-	gboolean          is_enabled;
-	gboolean          is_visible;
-
-  public:
-	MenuItemInfo ()
-	{
-		label = NULL;
-		check_state = DBUSMENU_MENUITEM_TOGGLE_STATE_UNKNOWN;
-		check_type  = (gchar*)DBUSMENU_MENUITEM_TOGGLE_CHECK;
-
-		is_visible = TRUE;		
-		is_enabled = TRUE;
-	}
-	~MenuItemInfo ()
-	{
-		if (label)
-			g_free(label);
-	}
-	
-	//Setters
-	void
-	setLabel (gchar* label)
-	{
-		this->label = g_strdup (label);
-	}
-	
-	void
-	setEnabled (gboolean is_enabled)
-	{
-		this->is_enabled = is_enabled;
-	}
-
-	void
-	setCheckState (gint check_state)
-	{
-		this->check_state = check_state;
-	}
-	
-	void
-	setCheckType (const gchar* check_type)
-	{
-		this->check_type = (gchar*)check_type;
-	}
-	
-	void
-	setVisible (gboolean is_visible)
-	{
-		this->is_visible = is_visible;
-	}
-	
-	//Getters
-	gchar*
-	getLabel ()
-	{
-		return label;
-	}
-	
-	gboolean
-	getEnabled ()
-	{
-		return is_enabled;
-	}
-
-	gint
-	getCheckState ()
-	{
-		return check_state;
-	}
-	
-	const gchar*
-	getCheckType ()
-	{
-		return check_type;
-	}
-	
-	gboolean
-	getVisible ()
-	{
-		return is_visible;
-	}
-};
-
 void
 destroy_menu_item_info (gpointer data)
 {
 	delete (MenuItemInfo*)data;
 }
 
-class MenuItemStatusListener : public cppu::WeakImplHelper1 < XStatusListener >
-{
-  private:
-	guint16 id;
-	FrameHelper *helper;	
-
-  public:
-  
-	MenuItemStatusListener (FrameHelper *helper)
-	{
-		if (!helper) throw ("FrameHelper cannot be NULL");
-		this->helper = helper;
-	}
-	
-	~MenuItemStatusListener ()
-	{
-	}
-	
-	virtual void SAL_CALL
-	statusChanged(const FeatureStateEvent& Event)
-	  throw (RuntimeException)
-	{	
-		sal_Bool            isChecked;
-		Visibility          visible;
-		OUString url = Event.FeatureURL.Complete;
-		OUString oULabel;
-		gchar* c_url = g_utf16_to_utf8 (url.getStr(),
-			                            url.getLength(),
-			                            NULL, NULL, NULL);
-		//g_debug ("UPDATE %s", c_url);
-		
-		GHashTable *commandsInfo = helper->getCommandsInfo ();
-		MenuItemInfo *info = (MenuItemInfo*)g_hash_table_lookup (commandsInfo, (gpointer)c_url);
-		if (!info)
-		{
-			info = new MenuItemInfo ();
-			g_hash_table_insert (commandsInfo, c_url, info);
-			
-			//Set the default label
-			oULabel = helper->getLabelFromCommandURL(url);
-			// Replace tilde with underscore for Dbusmenu Alt accelerators
-			oULabel = oULabel.replace ((sal_Unicode)0x007e, (sal_Unicode)0x005f);
-	
-			// GLib behaves better than OUStringToOString wrt encoding transformation
-			gchar* label = g_utf16_to_utf8 (oULabel.getStr(),
-			                                oULabel.getLength(),
-			                                NULL, NULL, NULL);		
-			info->setLabel (label);
-			g_free (label);
-		}
-		else
-		{
-			//Since we're not introducing it in the hash table, we get rid of this string
-			g_free (c_url);
-		}
-		
-		// For some reason, URLs can slip through as labels?
-		info->setEnabled ((gboolean)Event.IsEnabled);
-		
-		if ((Event.State >>= oULabel) &&
-		    !oULabel.matchAsciiL ("private:", 8, 0) &&
-		    !oULabel.matchAsciiL (".uno:", 5, 0)    &&
-		    !oULabel.matchAsciiL ("slot:", 5, 0)    &&
-		    !oULabel.matchAsciiL ("service:", 8, 0) &&
-		    !oULabel.matchAsciiL (".cmd:", 5, 0)    &&
-		    !oULabel.matchAsciiL ("macro:///", 5, 0))
-		{	
-			oULabel = oULabel.replace ((sal_Unicode)0x007e, (sal_Unicode)0x005f);
-			gchar* label = g_utf16_to_utf8 (oULabel.getStr(),
-		                             oULabel.getLength(),
-		                             NULL, NULL, NULL);		
-			info->setLabel (label);
-			g_free (label);
-		}
-		else if (Event.State >>= isChecked)
-		{
-			info->setCheckState (isChecked);
-		}
-		else if (Event.State >>= visible)
-		{
-			info->setVisible (visible.bVisible);
-		}
-		else 
-		{
-		}
-	}
-	
-	virtual void SAL_CALL disposing(const EventObject& aEvent)
-	  throw (RuntimeException)
-	{}
-};
-
+// ------------------------ FrameHelper Class -------------------------------
 FrameHelper::FrameHelper(const Reference< XMultiServiceFactory >&  rServiceManager,
                          const Reference< XFrame >&        xFrame,
                          DbusmenuServer*                   server)
@@ -378,11 +210,7 @@ FrameHelper::FrameHelper(const Reference< XMultiServiceFactory >&  rServiceManag
     xTrans = Reference < XURLTransformer > (xMSF->createInstance( rtl::OUString::createFromAscii("com.sun.star.util.URLTransformer" )), UNO_QUERY);
 	
 	xSL = (XStatusListener*)new MenuItemStatusListener (this);
-	commandsInfo = g_hash_table_new_full (g_str_hash,
-		                                  g_str_equal,
-		                                  g_free,
-		                                  destroy_menu_item_info);
-		                                  
+			                                  
 	// This initializes the shortcut database
 	getAcceleratorConfigurations (xFrame->getController()->getModel (), xMM);
 
@@ -390,9 +218,16 @@ FrameHelper::FrameHelper(const Reference< XMultiServiceFactory >&  rServiceManag
 	xPCF = Reference < XMultiComponentFactory > (xMSF->createInstance(OUString::createFromAscii("com.sun.star.frame.PopupMenuControllerFactory")),
 	                                             UNO_QUERY);
 	
-	args = Sequence < Any > (2);
-	//TODO: The frame identifier should be updated onLoad
-	
+
+	// This is a hash table that maps Command URLs to MenuItemInfo classes
+	//   to cache command information
+	commandsInfo = g_hash_table_new_full (g_str_hash,
+		                                  g_str_equal,
+		                                  g_free,
+		                                  destroy_menu_item_info);
+
+	// These are the arguments needed for the XPopupMenuController
+	args = Sequence < Any > (2);	
 	PropertyValue item;
 	
 	item.Name = OUString::createFromAscii("ModuleName");
@@ -405,6 +240,10 @@ FrameHelper::FrameHelper(const Reference< XMultiServiceFactory >&  rServiceManag
 	
 	root = NULL;
 	watcher_set = FALSE;
+
+	//This variable prevents the helper from being disconnected from the frame
+	//for special cases of component dettaching like print preview
+	blockDetach = FALSE;
 }
 
 void SAL_CALL 
@@ -477,14 +316,380 @@ FrameHelper::getXID ()
 
 	return xWindow.WindowHandle;
 }
+
+void SAL_CALL
+FrameHelper::frameAction(const FrameActionEvent& action) throw (RuntimeException)
+{
+	//If theh component is detached from the frame, remove this action listener,
+	//it is then disposed and destroyed by the frame. We deregister the window
+	//from the AppMenu Registrar
+
+	//This is a special case, .uno:printPreview detaches the component but we are
+	//not actually switching to another document.
+	if (blockDetach)
+	{
+		blockDetach = TRUE;
+		return;
+	}
+
+	if (action.Action == frame::FrameAction_COMPONENT_DETACHING)
+	{
+		GError *error = NULL;
+		
+		
+		xFrame->removeFrameActionListener (this);
+		Reference< XPropertySet > frameProps (xFrame, UNO_QUERY);
+		Reference < XLayoutManager > xLayoutManager(frameProps->getPropertyValue(OUString::createFromAscii("LayoutManager")),
+		                                            UNO_QUERY);
+		xLayoutManager->showElement (OUString::createFromAscii("private:resource/menubar/menubar"));
+
+		unsigned long xid = getXID();
+
+ 		GDBusProxy *proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+		                                                   G_DBUS_PROXY_FLAGS_NONE,
+		                                                   NULL,
+		                                                   "com.canonical.AppMenu.Registrar",
+		                                                   "/com/canonical/AppMenu/Registrar",
+		                                                   "com.canonical.AppMenu.Registrar",
+ 		                                                   NULL,
+ 		                                                   &error);
+		if (error)
+		{
+			g_warning ("Couldn't get /com/canonical/AppMenu/Registrar proxy");
+			g_error_free (error);
+			return;
+		}
+
+		//TODO: Check if window is registered already
+		g_dbus_proxy_call_sync (proxy,
+			                    "UnregisterWindow",
+			                    g_variant_new ("(u)", (guint32)xid),
+			                    G_DBUS_CALL_FLAGS_NONE,
+			                    -1,
+			                    NULL,
+			                    &error);
 	
+		if (error)
+		{
+			g_warning ("Couldn't call /com/canonical/AppMenu/Registrar.UnregisterWindow");
+			g_error_free (error);
+		}
+		
+		if (server)
+		{
+			g_object_unref (server);
+			server = NULL;
+			root = NULL;
+		}
+
+		if (watcher_set)
+		{
+			g_bus_unwatch_name (watcher);
+			watcher_set = FALSE;
+		}
+		
+		return;
+	}
+}
+
+//This function rebuilds (or builds from scratch) a DbusmenuMenuitem structure
+//from a given pair of XMenu/Dbusmenuitem.
+void
+FrameHelper::rebuildMenu (Reference < XMenu >  xMenu,
+                          DbusmenuMenuitem    *parent)
+{
+	g_return_if_fail (parent != NULL);
+	GList *items = dbusmenu_menuitem_get_children (parent);	
+	guint nitems = g_list_length (items);   //number of available Dbusmenuitems
+	guint16 count = xMenu->getItemCount (); //number of real menu items
+	
+	// One item does not represent always the same command.
+	// We do this for performance reasons, as it's really hard to match a command with
+	// a single dbusmenuitem given the lack of information provided by the status listener
+	if (count > nitems)
+	{
+		// Add enough Dbusmenuitems to replicate all 
+		for (guint16 i = 0; i < (count - nitems); i++)
+		{
+			DbusmenuMenuitem *item = dbusmenu_menuitem_new ();
+			dbusmenu_menuitem_child_append (parent, item);
+			g_signal_connect(G_OBJECT(item), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(item_activated), this);
+		}
+		items = dbusmenu_menuitem_get_children (parent);	
+	}
+	if (count < nitems)
+	{
+		// If there is an excess of Dbusmenuitems we make them invisible
+		for (guint16 i = nitems - 1; i >= count; i--)
+		{
+			DbusmenuMenuitem *item = DBUSMENU_MENUITEM (g_list_nth_data(items, i));
+			dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
+		}
+	}
+	
+	for (guint16 i = 0; i<count; i++)
+	{
+		Reference < XMenuExtended > xMenuEx (xMenu, UNO_QUERY);
+		guint16 id = xMenu->getItemId (i);
+		OUString oUCommand = xMenuEx->getCommand (id);
+		OString command = OUStringToOString (oUCommand, RTL_TEXTENCODING_ASCII_US);
+		
+		DbusmenuMenuitem *item = DBUSMENU_MENUITEM(g_list_nth_data(items, i));
+				
+		if (!item)
+			continue;
+
+		if (!DBUSMENU_IS_MENUITEM (item))
+			continue;
+		
+		// We drop the WindowList, doesn't work properly and it's useless anyhow
+		if (oUCommand.equals (OUString::createFromAscii (".uno:WindowList")))
+			continue;
+		
+		//We set the default properties (in case it was not visible or a separator)
+		dbusmenu_menuitem_property_set (item, DBUSMENU_MENUITEM_PROP_TYPE, DBUSMENU_CLIENT_TYPES_DEFAULT);
+		dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_VISIBLE, TRUE);
+
+		if (id == 0)
+		{
+       		dbusmenu_menuitem_property_set (item, "CommandURL", "slot:0");
+            dbusmenu_menuitem_property_set (item, DBUSMENU_MENUITEM_PROP_TYPE, DBUSMENU_CLIENT_TYPES_SEPARATOR);
+			//Getting rid of any possible children
+   			g_list_free_full (dbusmenu_menuitem_take_children (item), destroy_menuitem);
+       		continue;
+		}
+		
+		//Setting the command
+		dbusmenu_menuitem_property_set (item, "CommandURL", command.getStr());
+
+		//Getting a shortcut
+		KeyEvent kev = findShortcutForCommand (oUCommand);
+
+		if (kev.KeyCode != 0)  //KeyCode must have a value
+		{
+			GVariantBuilder builder;
+			const gchar* keystring = AwtKeyToDbusmenuString(kev.KeyCode);
+
+			g_variant_builder_init(&builder, G_VARIANT_TYPE_ARRAY);
+
+			//We map KeyEvent.Modifiers with Dbusmenu modifiers strings
+			if (awt::KeyModifier::SHIFT & kev.Modifiers)
+				g_variant_builder_add(&builder, "s", DBUSMENU_MENUITEM_SHORTCUT_SHIFT);
+			if (awt::KeyModifier::MOD2 & kev.Modifiers)
+				g_variant_builder_add(&builder, "s", DBUSMENU_MENUITEM_SHORTCUT_ALT);
+			if (awt::KeyModifier::MOD1 & kev.Modifiers || awt::KeyModifier::MOD3 & kev.Modifiers)
+				g_variant_builder_add(&builder, "s", DBUSMENU_MENUITEM_SHORTCUT_CONTROL);
+
+			g_variant_builder_add(&builder, "s", keystring);
+
+			GVariant * inside = g_variant_builder_end(&builder);
+			g_variant_builder_init(&builder, G_VARIANT_TYPE_ARRAY);
+			g_variant_builder_add_value(&builder, inside);
+
+			GVariant * outsidevariant = g_variant_builder_end(&builder);
+			dbusmenu_menuitem_property_set_variant(item, DBUSMENU_MENUITEM_PROP_SHORTCUT, outsidevariant);
+		}
+
+		// Lookup for a MenuItemInfo object for this menuitem, create one if it doesn't exist
+		// this object caches the values that change on status updates.
+		MenuItemInfo* commInfo = (MenuItemInfo*)g_hash_table_lookup (commandsInfo, (gconstpointer)command.getStr());
+		if (!commInfo)
+		{
+			commInfo = new MenuItemInfo ();
+			g_hash_table_insert (commandsInfo, g_strdup (command.getStr()), commInfo);
+
+			OUString oULabel = getLabelFromCommandURL(oUCommand);
+			if (oULabel.getLength() == 0)
+			{
+				oULabel = xMenu->getItemText (id);
+			}
+			
+			//Replace tilde with underscore for Dbusmenu Alt accelerators
+			oULabel = oULabel.replace ((sal_Unicode)0x007e, (sal_Unicode)0x005f);
+			// GLib behaves better than OUStringToOString wrt encoding transformation
+			gchar* label = g_utf16_to_utf8 (oULabel.getStr(),
+			                                oULabel.getLength(),
+			                                NULL, NULL, NULL);
+			commInfo->setLabel (label);
+			g_free (label);
+		}
+
+		//Update the check state directly from the data, this is more reliable
+		Reference < XPopupMenu > popUp (xMenu, UNO_QUERY);
+		if (popUp.is() && popUp->isItemChecked (id))
+		{
+			commInfo->setCheckState (DBUSMENU_MENUITEM_TOGGLE_STATE_CHECKED);
+		}
+		
+		dbusmenu_menuitem_property_set (item, DBUSMENU_MENUITEM_PROP_LABEL, commInfo->getLabel ());
+		dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_ENABLED, commInfo->getEnabled ());
+
+		//TODO: Find a selection of which commands are radio toggle type
+		if (commInfo->getCheckState () != DBUSMENU_MENUITEM_TOGGLE_STATE_UNKNOWN)
+		{
+			dbusmenu_menuitem_property_set (item, DBUSMENU_MENUITEM_PROP_TOGGLE_TYPE,  commInfo->getCheckType ());
+			dbusmenu_menuitem_property_set_int (item, DBUSMENU_MENUITEM_PROP_TOGGLE_STATE, commInfo->getCheckState ());
+		}
+
+		// Adding status listener
+		URL commandURL;
+		commandURL.Complete = oUCommand;
+		xTrans->parseStrict (commandURL);
+
+		Reference < XDispatch > xDispatch  = xdp->queryDispatch (commandURL, OUString(), 0);
+		if (xDispatch.is())
+			xDispatch->addStatusListener (xSL, commandURL);
+
+		Reference < XPopupMenu > subPopMenu (xMenu->getPopupMenu (id), UNO_QUERY);
+		
+		//Some menus do not provide the information available through the normal XMenu interface,
+		//we need to access that info through a special XPopupMenuController
+		if (isSpecialSubmenu (oUCommand))
+		{
+			Reference < XPropertySet > xMSFProps (xMSF, UNO_QUERY);
+			Reference <XComponentContext> xContext (xMSFProps->getPropertyValue (OUString::createFromAscii ("DefaultContext")),
+			                                        UNO_QUERY);
+
+			Reference < XPopupMenuController > xRFC (xPCF->createInstanceWithArgumentsAndContext(oUCommand,
+	                                                                                             args,
+                                                                                                 xContext),
+                                                     UNO_QUERY);
+
+			Reference < XPopupMenu > xPO (xMSF->createInstance(OUString::createFromAscii ("stardiv.Toolkit.VCLXPopupMenu")),
+			                              UNO_QUERY);
+
+			if (xRFC.is () && xPO.is ())
+			{
+				xRFC->setPopupMenu (xPO);
+				xRFC->updatePopupMenu ();
+				Reference < XMenu > subMenu (xPO, UNO_QUERY);
+				rebuildMenu (subMenu, item);
+			}
+			else if (subPopMenu.is ())
+			{
+				Reference <XMenu> subMenu (subPopMenu, UNO_QUERY);
+				rebuildMenu (subMenu, item);			
+			}
+		}
+
+		// Introspect submenus
+		else if (subPopMenu.is ())
+		{
+			Reference <XMenu> subMenu (subPopMenu, UNO_QUERY);
+			g_signal_connect(G_OBJECT(item), DBUSMENU_MENUITEM_SIGNAL_ABOUT_TO_SHOW, G_CALLBACK(item_about_to_show), this);
+			rebuildMenu (subMenu, item);
+		}
+		else
+		{
+			//Getting rid of any possible children
+			g_list_free_full (dbusmenu_menuitem_take_children (item), destroy_menuitem);
+		}		
+	}
+
+	return;
+}
+
+//Gets the menu item Label given a CommandURL
+//This is a work around for bug: https://bugs.freedesktop.org/show_bug.cgi?id=34127
+OUString
+FrameHelper::getLabelFromCommandURL (OUString commandURL)
+{
+	OUString label;
+
+	Sequence < PropertyValue > commandProps;
+	
+	if (commandURL.getLength () < 1)
+		return label;
+	
+	if (!xUICommands.is())
+		return label;
+	
+	try
+	{
+		xUICommands->getByName (commandURL) >>= commandProps;
+	}
+	catch (com::sun::star::container::NoSuchElementException e)
+	{
+		return label;
+	}
+	
+	for (sal_Int32 i = 0; i < commandProps.getLength(); i++)
+	{
+		if ( commandProps[i].Name.equalsAsciiL (RTL_CONSTASCII_STRINGPARAM ("Label")))
+		{
+			commandProps[i].Value >>= label;
+			label = label.replace ((sal_Unicode)0x007e, (sal_Unicode)0x005f);
+			//break;
+		}
+	}
+
+	return label;
+}
+
+//This method is a facility to bootstrap the Dbusmenuitem strcuture from the menubar
+void
+FrameHelper::rebuildMenuFromRoot ()
+{
+	Reference < XFrame >         xFrame = getFrame ();
+	Reference < XPropertySet >   frameProps     (xFrame, UNO_QUERY);
+	Reference < XLayoutManager > xLayoutManager (frameProps->getPropertyValue(OUString::createFromAscii("LayoutManager")),
+	                                             UNO_QUERY);
+	Reference < XUIElement >   menuBar     (xLayoutManager->getElement (OUString::createFromAscii("private:resource/menubar/menubar")),
+	                                        UNO_QUERY);
+	Reference < XPropertySet > menuPropSet (menuBar, UNO_QUERY);
+	
+	if (!menuPropSet.is ())
+		return;
+	
+	Reference < XMenu > xMenu (menuPropSet->getPropertyValue(OUString::createFromAscii("XMenuBar")),
+	                           UNO_QUERY);
+	if (!xMenu.is ())
+		return;
+
+	rebuildMenu (xMenu, root);
+}
+
+//Some menus are special, this is the list of them
+gboolean
+FrameHelper::isSpecialSubmenu (OUString command)
+{
+	const gchar * specialSubmenus[11] = {".uno:CharFontName",
+	                                     ".uno:FontHeight",
+	                                     ".uno:ObjectMenue",
+	                                     ".uno:InsertPageHeader",
+	                                     ".uno:InsertPageFooter",
+	                                     ".uno:ChangeControlType",
+	                                     ".uno:AvailableToolbars",
+	                                     ".uno:ScriptOrganizer",
+	                                     ".uno:RecentFileList",
+	                                     ".uno:AddDirect",
+	                                     ".uno:AutoPilotMenu"};
+
+	for (gint i = 0; i < 11; i++)
+	{
+		if (command.equals (OUString::createFromAscii (specialSubmenus[i])))
+			return TRUE;
+	}
+	return FALSE;
+}
+
 void
 FrameHelper::dispatchCommand (OUString command)
 {
 	OUString target = OUString::createFromAscii ("");
 	Reference < XDispatchHelper > xdh (xMSF->createInstance(OUString(RTL_CONSTASCII_USTRINGPARAM("com.sun.star.frame.DispatchHelper"))),
 	                                   UNO_QUERY);
-	// This is a special case for the recentfilelist
+
+	//g_debug ("%s", OUStringToOString (command, RTL_TEXTENCODING_ASCII_US).getStr());
+
+	// This is a special case, we don't want the helper to be disconnected from the frame
+	// when PrintPreview dettaches. See the frameAction method.
+	if (command.equals (OUString::createFromAscii (".uno:PrintPreview")))
+	{
+		blockDetach = TRUE;
+	}
+	
+	// This is a special case for the recentfilelist	
 	if (command.matchAsciiL ("vnd.sun.star.popup:RecentFileList", 33, 0))
 	{
 	    target = OUString::createFromAscii ("_default");
@@ -526,10 +731,9 @@ FrameHelper::dispatchCommand (OUString command)
 		return;
 	}
 
-	
 	if (command.matchAsciiL ("private:factory/", 16, 0))
 	    target = OUString::createFromAscii ("_blank");
-	
+
 	xdh->executeDispatch (Reference < XDispatchProvider > (xFrame, UNO_QUERY),
 	                      command,
 	                      target,
@@ -537,325 +741,6 @@ FrameHelper::dispatchCommand (OUString command)
 	                      Sequence < PropertyValue > ());
 }
 
-
-void SAL_CALL
-FrameHelper::frameAction(const FrameActionEvent& action) throw (RuntimeException)
-{
-	//We deregister this action listener, it is then disposed and the dbusmenuserver shutdown	
-	if (action.Action == frame::FrameAction_COMPONENT_DETACHING)
-	{
-		GError *error = NULL;
-		xFrame->removeFrameActionListener (this);
-		Reference< XPropertySet > frameProps (xFrame, UNO_QUERY);
-		Reference < XLayoutManager > xLayoutManager(frameProps->getPropertyValue(OUString::createFromAscii("LayoutManager")),
-		                                            UNO_QUERY);
-		xLayoutManager->showElement (OUString::createFromAscii("private:resource/menubar/menubar"));
-		
-		unsigned long xid = getXID();
-	
- 		GDBusProxy *proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
-		                                                   G_DBUS_PROXY_FLAGS_NONE,
-		                                                   NULL,
-		                                                   "com.canonical.AppMenu.Registrar",
-		                                                   "/com/canonical/AppMenu/Registrar",
-		                                                   "com.canonical.AppMenu.Registrar",
- 		                                                   NULL,
- 		                                                   &error);
-		if (error)
-		{
-			g_warning ("Couldn't get /com/canonical/AppMenu/Registrar proxy");
-			g_error_free (error);
-			return;
-		}
-
-
-		//TODO: Check if window is registered already
-		g_dbus_proxy_call_sync (proxy,
-			                    "UnregisterWindow",
-			                    g_variant_new ("(u)", (guint32)xid),
-			                    G_DBUS_CALL_FLAGS_NONE,
-			                    -1,
-			                    NULL,
-			                    &error);
-	
-		if (error)
-		{
-			g_warning ("Couldn't call /com/canonical/AppMenu/Registrar.UnregisterWindow");
-			g_error_free (error);
-		}
-		
-		return;
-	}
-}
-
-
-
-void
-FrameHelper::rebuildMenu (Reference < XMenu >  xMenu,
-                          DbusmenuMenuitem    *parent)
-{
-	g_return_if_fail (parent != NULL);
-	GList *items = dbusmenu_menuitem_get_children (parent);	
-	guint nitems = g_list_length (items);
-	guint16 count = xMenu->getItemCount ();
-	
-	// One item does not represent always the same command.
-	// We do this for performance reasons, as it's really hard to match a command with
-	// a single dbusmenuitem given the lack of information provided by the status listener
-	if (count > nitems)
-	{
-		// Add enough items to replicate all 
-		for (guint16 i = 0; i < (count - nitems); i++)
-		{
-			DbusmenuMenuitem *item = dbusmenu_menuitem_new ();
-			dbusmenu_menuitem_child_append (parent, item);
-			g_signal_connect(G_OBJECT(item), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(item_activated), this);
-		}
-		items = dbusmenu_menuitem_get_children (parent);	
-	}
-	if (count < nitems)
-	{
-		for (guint16 i = nitems - 1; i >= count; i--)
-		{
-			DbusmenuMenuitem *item = DBUSMENU_MENUITEM (g_list_nth_data(items, i));
-			dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
-		}
-	}
-	
-	for (guint16 i = 0; i<count; i++)
-	{
-		Reference < XMenuExtended > xMenuEx (xMenu, UNO_QUERY);
-		guint16 id = xMenu->getItemId (i);
-		OUString oUCommand = xMenuEx->getCommand (id);
-		OString command = OUStringToOString (oUCommand, RTL_TEXTENCODING_ASCII_US);
-		
-		DbusmenuMenuitem *item = DBUSMENU_MENUITEM(g_list_nth_data(items, i));
-				
-		if (!item)
-			continue;
-
-		if (!DBUSMENU_IS_MENUITEM (item))
-			continue;
-		
-		// We drop the WindowList, it's useless and doesn't work properly
-		if (oUCommand.equals (OUString::createFromAscii (".uno:WindowList")))
-			continue;
-		
-		//We set the default properties (in case it was not visible or a separator)
-		dbusmenu_menuitem_property_set (item, DBUSMENU_MENUITEM_PROP_TYPE, DBUSMENU_CLIENT_TYPES_DEFAULT);
-		dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_VISIBLE, TRUE);
-
-		if (id == 0)
-		{
-       		dbusmenu_menuitem_property_set (item, "CommandURL", "slot:0");
-            dbusmenu_menuitem_property_set (item, DBUSMENU_MENUITEM_PROP_TYPE, DBUSMENU_CLIENT_TYPES_SEPARATOR);
-			//Getting rid of any possible children
-   			g_list_free_full (dbusmenu_menuitem_take_children (item), destroy_menuitem);
-       		continue;
-		}
-		
-		//Setting the command
-		dbusmenu_menuitem_property_set (item, "CommandURL", command.getStr());
-
-		//Getting a shortcut
-		KeyEvent kev = findShortcutForCommand (oUCommand);
-		//g_debug ("%s: %x - %x - %x - %x", command.getStr(), kev.KeyCode, kev.KeyChar, kev.KeyFunc, kev.Modifiers);
-
-		if (kev.KeyCode != 0)  //KeyCode must have a value
-		{
-			GVariantBuilder builder;
-			const gchar* keystring = AwtKeyToDbusmenuString(kev.KeyCode);
-
-			g_variant_builder_init(&builder, G_VARIANT_TYPE_ARRAY);
-
-			if (awt::KeyModifier::SHIFT & kev.Modifiers)
-				g_variant_builder_add(&builder, "s", DBUSMENU_MENUITEM_SHORTCUT_SHIFT);
-			if (awt::KeyModifier::MOD2 & kev.Modifiers)
-				g_variant_builder_add(&builder, "s", DBUSMENU_MENUITEM_SHORTCUT_ALT);
-			if (awt::KeyModifier::MOD1 & kev.Modifiers || awt::KeyModifier::MOD3 & kev.Modifiers)
-				g_variant_builder_add(&builder, "s", DBUSMENU_MENUITEM_SHORTCUT_CONTROL);
-
-			g_variant_builder_add(&builder, "s", keystring);
-
-			GVariant * inside = g_variant_builder_end(&builder);
-			g_variant_builder_init(&builder, G_VARIANT_TYPE_ARRAY);
-			g_variant_builder_add_value(&builder, inside);
-
-			GVariant * outsidevariant = g_variant_builder_end(&builder);
-			dbusmenu_menuitem_property_set_variant(item, DBUSMENU_MENUITEM_PROP_SHORTCUT, outsidevariant);
-		}
-
-		MenuItemInfo* commInfo = (MenuItemInfo*)g_hash_table_lookup (commandsInfo, (gconstpointer)command.getStr());
-		if (!commInfo)
-		{
-			commInfo = new MenuItemInfo ();
-			g_hash_table_insert (commandsInfo, g_strdup (command.getStr()), commInfo);
-			
-			
-			OUString oULabel = getLabelFromCommandURL(oUCommand);
-			if (oULabel.getLength() == 0)
-			{
-				oULabel = xMenu->getItemText (id);
-			}
-			
-			//Replace tilde with underscore for Dbusmenu Alt accelerators
-			oULabel = oULabel.replace ((sal_Unicode)0x007e, (sal_Unicode)0x005f);
-	
-			// GLib behaves better than OUStringToOString wrt encoding transformation
-			gchar* label = g_utf16_to_utf8 (oULabel.getStr(),
-			                                oULabel.getLength(),
-			                                NULL, NULL, NULL);
-			commInfo->setLabel (label);
-					
-			g_free (label);
-		}
-
-		//Update the check state directly from the data, this is more reliable
-
-		Reference < XPopupMenu > popUp (xMenu, UNO_QUERY);
-		if (popUp.is() && popUp->isItemChecked (id))
-		{
-			commInfo->setCheckState (DBUSMENU_MENUITEM_TOGGLE_STATE_CHECKED);
-		}
-		
-		dbusmenu_menuitem_property_set (item, DBUSMENU_MENUITEM_PROP_LABEL, commInfo->getLabel ());
-		dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_ENABLED, commInfo->getEnabled ());
-
-		if (commInfo->getCheckState () != DBUSMENU_MENUITEM_TOGGLE_STATE_UNKNOWN)
-		{
-			dbusmenu_menuitem_property_set (item, DBUSMENU_MENUITEM_PROP_TOGGLE_TYPE,  commInfo->getCheckType ());
-			dbusmenu_menuitem_property_set_int (item, DBUSMENU_MENUITEM_PROP_TOGGLE_STATE, commInfo->getCheckState ());
-		}
-
-		// Adding status listener
-		URL commandURL;
-		commandURL.Complete = oUCommand;
-		xTrans->parseStrict (commandURL);
-
-		Reference < XDispatch > xDispatch  = xdp->queryDispatch (commandURL, OUString(), 0);
-		if (xDispatch.is())
-			xDispatch->addStatusListener (xSL, commandURL);
-
-		Reference < XPopupMenu > subPopMenu (xMenu->getPopupMenu (id), UNO_QUERY);
-		
-		//Build special submenu information
-		if (isSpecialSubmenu (oUCommand))
-		{
-			Reference < XPropertySet > xMSFProps (xMSF, UNO_QUERY);
-			Reference <XComponentContext> xContext (xMSFProps->getPropertyValue (OUString::createFromAscii ("DefaultContext")),
-			                                        UNO_QUERY);
-
-			Reference < XPopupMenuController > xRFC (xPCF->createInstanceWithArgumentsAndContext(oUCommand,
-	                                                                                             args,
-                                                                                                 xContext),
-                                                     UNO_QUERY);
-
-			Reference < XPopupMenu > xPO (xMSF->createInstance(OUString::createFromAscii ("stardiv.Toolkit.VCLXPopupMenu")),
-			                              UNO_QUERY);
-			
-			if (xRFC.is () && xPO.is ())
-			{
-				xRFC->setPopupMenu (xPO);
-				xRFC->updatePopupMenu ();
-				Reference < XMenu > subMenu (xPO, UNO_QUERY);
-				rebuildMenu (subMenu, item);
-			}
-			else if (subPopMenu.is ())
-			{
-				Reference <XMenu> subMenu (subPopMenu, UNO_QUERY);
-				rebuildMenu (subMenu, item);			
-			}
-		}
-		// Introspect submenus
-		else if (subPopMenu.is ())
-		{
-			Reference <XMenu> subMenu (subPopMenu, UNO_QUERY);
-			g_signal_connect(G_OBJECT(item), DBUSMENU_MENUITEM_SIGNAL_ABOUT_TO_SHOW, G_CALLBACK(item_about_to_show), this);
-			rebuildMenu (subMenu, item);
-		}
-		else
-		{
-			//Getting rid of any possible children
-			g_list_free_full (dbusmenu_menuitem_take_children (item), destroy_menuitem);
-		}		
-	}
-
-	return;
-}
-
-//Gets the menu item Label given a CommandURL
-//This is a work around for bug: https://bugs.freedesktop.org/show_bug.cgi?id=34127
-OUString
-FrameHelper::getLabelFromCommandURL (OUString commandURL)
-{
-	OUString label;
-
-	Sequence < PropertyValue > commandProps;
-	
-	if (commandURL.getLength () < 1)
-		return label;
-	
-	try
-	{
-		xUICommands->getByName (commandURL) >>= commandProps;
-	}
-	catch (com::sun::star::container::NoSuchElementException e)
-	{
-		return label;
-	}
-	
-	for (sal_Int32 i = 0; i < commandProps.getLength(); i++)
-	{
-		if ( commandProps[i].Name.equalsAsciiL (RTL_CONSTASCII_STRINGPARAM ("Label")))
-		{
-			commandProps[i].Value >>= label;
-			label = label.replace ((sal_Unicode)0x007e, (sal_Unicode)0x005f);
-			//break;
-		}
-	}
-
-	return label;
-}
-
-void
-FrameHelper::rebuildMenuFromRoot ()
-{
-	Reference < XFrame >         xFrame = getFrame ();
-	Reference < XPropertySet >   frameProps     (xFrame, UNO_QUERY);
-	Reference < XLayoutManager > xLayoutManager (frameProps->getPropertyValue(OUString::createFromAscii("LayoutManager")),
-	                                             UNO_QUERY);
-	Reference < XUIElement >   menuBar     (xLayoutManager->getElement (OUString::createFromAscii("private:resource/menubar/menubar")),
-	                                        UNO_QUERY);
-	Reference < XPropertySet > menuPropSet (menuBar, UNO_QUERY);
-	
-	Reference < XMenu > xMenu (menuPropSet->getPropertyValue(OUString::createFromAscii("XMenuBar")),
-	                           UNO_QUERY);
-	rebuildMenu (xMenu, root);
-}
-
-gboolean
-FrameHelper::isSpecialSubmenu (OUString command)
-{
-	const gchar * specialSubmenus[11] = {".uno:CharFontName",
-	                                     ".uno:FontHeight",
-	                                     ".uno:ObjectMenue",
-	                                     ".uno:InsertPageHeader",
-	                                     ".uno:InsertPageFooter",
-	                                     ".uno:ChangeControlType",
-	                                     ".uno:AvailableToolbars",
-	                                     ".uno:ScriptOrganizer",
-	                                     ".uno:RecentFileList",
-	                                     ".uno:AddDirect",
-	                                     ".uno:AutoPilotMenu"};
-
-	for (gint i = 0; i < 11; i++)
-	{
-		if (command.equals (OUString::createFromAscii (specialSubmenus[i])))
-			return TRUE;
-	}
-	return FALSE;
-}
-
 //Set all the accelerator configuration sources
 void
 FrameHelper::getAcceleratorConfigurations (Reference < XModel >        xModel,
@@ -880,6 +765,8 @@ FrameHelper::getAcceleratorConfigurations (Reference < XModel >        xModel,
 	this->globAccelConf = globAccelConf;
 }
 
+
+//This function finds a KeyEvent with the shortcut information for each command
 KeyEvent
 FrameHelper::findShortcutForCommand (OUString command)
 
@@ -888,7 +775,7 @@ FrameHelper::findShortcutForCommand (OUString command)
 
 	Sequence < OUString > commands (1);
 	commands[0] = command;
-	
+
 	try 
 	{
 		Sequence < Any > evs = docAccelConf->getPreferredKeyEventsForCommandList (commands);
@@ -905,7 +792,6 @@ FrameHelper::findShortcutForCommand (OUString command)
 	try 
 	{
 		Sequence < Any > evs = modAccelConf->getPreferredKeyEventsForCommandList (commands);
-		
 
 		for (int j = 0; j < evs.getLength (); j++)
 		{
@@ -919,7 +805,7 @@ FrameHelper::findShortcutForCommand (OUString command)
 	try 
 	{
 		Sequence < Any > evs = globAccelConf->getPreferredKeyEventsForCommandList (commands);
-		
+
 		for (int j = 0; j < evs.getLength (); j++)
 		{
 			KeyEvent ev;
