@@ -20,6 +20,9 @@
  * version 3 along with this program. If not, see <http://www.gnu.org/licenses/>
  *
  */
+ 
+#ifndef __FRAME_HELPER_H__
+#define __FRAME_HELPER_H__
 
 #include <com/sun/star/awt/KeyEvent.hpp>
 #include <com/sun/star/awt/XMenu.hpp>
@@ -84,6 +87,7 @@ class FrameHelper : public cppu::WeakImplHelper1 < XFrameActionListener >
 	Reference < XURLTransformer >       xTrans;
 	Reference < XDispatchProvider >     xdp;
 	GHashTable                         *commandsInfo;
+	gboolean                            blockDetach;
 
 	//These object/methods are used to recreate dynamic popupmenus
 	Reference < XMultiComponentFactory > xPCF;
@@ -132,3 +136,4 @@ class FrameHelper : public cppu::WeakImplHelper1 < XFrameActionListener >
 	void rebuildMenuFromRoot ();
 	void populateWindowList  (DbusmenuMenuitem *);
 };
+#endif // __FRAME_HELPER_H__
