@@ -25,6 +25,8 @@
 #include "DesktopJob.h"
 #include "FrameHelper.h"
 
+#define OBJ_PATH_PREFIX "/com/canonical/menu/"
+
 #include <com/sun/star/awt/XSystemDependentWindowPeer.hpp>
 #include <com/sun/star/awt/SystemDependentXWindow.hpp>
 #include <com/sun/star/awt/XMenu.hpp>
@@ -66,8 +68,6 @@
 #include <rtl/process.h>
 #include <osl/diagnose.h>
 
-#define OBJ_PATH_PREFIX "/com/canonical/menu/"
-
 using rtl::OUString;
 using rtl::OString;
 using rtl::OUStringToOString;
@@ -131,7 +131,6 @@ xid_to_object_path (unsigned long xid)
 
 	GString *xid_str = g_string_new ("");
 	g_string_printf (xid_str, "%d", (guint32)xid);
-
 	OString object_path = OUStringToOString (OUString::createFromAscii (OBJ_PATH_PREFIX).concat (OUString::createFromAscii(xid_str->str)),
 	                                         RTL_TEXTENCODING_ASCII_US);
 	g_string_free (xid_str, TRUE);
@@ -190,7 +189,7 @@ on_registrar_available (GDBusConnection *connection,
 	Reference < XLayoutManager > xLayoutManager(frameProps->getPropertyValue(OUString::createFromAscii("LayoutManager")),
 	                                            UNO_QUERY);
 	xLayoutManager->hideElement (OUString::createFromAscii("private:resource/menubar/menubar"));
-	
+
 	return;
 }
 
@@ -211,7 +210,7 @@ on_registrar_unavailable (GDBusConnection *connection,
 	xLayoutManager->showElement (OUString::createFromAscii("private:resource/menubar/menubar"));
 	return;
 }
-
+// ------------------------------- FrameJob --------------------------------------------
 Any SAL_CALL FrameJob::execute( const Sequence< NamedValue >& aArguments )
 	throw ( IllegalArgumentException, Exception, RuntimeException )
 {	
@@ -250,13 +249,11 @@ Any SAL_CALL FrameJob::execute( const Sequence< NamedValue >& aArguments )
 	if (!xFrame.is ())
 		return Any();
 	
-	
-	
 	exportMenus	(xFrame);
 	return Any();
 }
 
-
+// This function crates a DbusmenuServer and starts the watcher for the AppMenu Registrar bus name
 void
 FrameJob::exportMenus (Reference < XFrame > xFrame)
 {	
@@ -280,6 +277,12 @@ FrameJob::exportMenus (Reference < XFrame > xFrame)
 	Reference < XUIElement > menuBar(xLayoutManager->getElement (OUString::createFromAscii("private:resource/menubar/menubar")),
 	                                 UNO_QUERY);
 	Reference < XPropertySet > menuPropSet (menuBar, UNO_QUERY);
+	
+	if (!menuPropSet.is())
+	{
+		g_object_unref (server);
+		return;
+	}
 
 	Reference < XMenu > xMenu (menuPropSet->getPropertyValue(OUString::createFromAscii("XMenuBar")),
 	                           UNO_QUERY);
@@ -293,14 +296,11 @@ FrameJob::exportMenus (Reference < XFrame > xFrame)
 	//Create a new frame helper	to close the server when needed
 	FrameHelper *helper = new FrameHelper (mxMSF, xFrame, server);
 	xFrame->addFrameActionListener (Reference < XFrameActionListener > (helper));
-	
-	xLayoutManager->hideElement (OUString::createFromAscii("private:resource/menubar/menubar"));
 
 	//Populate dbusmenu items and start the server
 	DbusmenuMenuitem *root = getRootMenuitem (xMenu, (gpointer)helper);
 	dbusmenu_server_set_root (server, root);
-	
-	
+
 	//Listen to the availability of the registrar
 	guint watcher = g_bus_watch_name (G_BUS_TYPE_SESSION,
 	                                  "com.canonical.AppMenu.Registrar",
@@ -312,6 +312,39 @@ FrameJob::exportMenus (Reference < XFrame > xFrame)
 	helper->setRegistrarWatcher (watcher);
 }
 
+
+//Gets the XID for a given XFrame
+unsigned long
+FrameJob::getXID (css::uno::Reference < css::frame::XFrame > xFrame)
+{
+	Reference< XSystemDependentWindowPeer > xWin( xFrame->getContainerWindow(), UNO_QUERY);
+
+	if (!xWin.is())
+		return 0;
+
+	sal_Int8 processID[16];
+	rtl_getGlobalProcessId( (sal_uInt8*)processID );
+	Sequence <signed char> pidSeq (processID, 16);
+
+	SystemDependentXWindow xWindow;
+	xWin->getWindowHandle (pidSeq, SYSTEM_XWINDOW) >>= xWindow;
+	
+	return xWindow.WindowHandle;
+}
+
+// Builds a Dbusmenuitem structure from an XMenu object
+DbusmenuMenuitem*
+FrameJob::getRootMenuitem (Reference < XMenu > xMenu, gpointer helper)
+{
+	
+	DbusmenuMenuitem *root = dbusmenu_menuitem_new_with_id (0);
+	((FrameHelper*)helper)->setRootItem(root);
+	((FrameHelper*)helper)->rebuildMenu (xMenu, root);
+	
+	return root;
+}
+
+// XJob
 OUString FrameJob_getImplementationName ()
 	throw (RuntimeException)
 {
@@ -357,34 +390,4 @@ Sequence< OUString > SAL_CALL FrameJob::getSupportedServiceNames()
 {
     return FrameJob_getSupportedServiceNames();
 }
-//----------- Private functions -------------
 
-//Gets the XID for a given XFrame
-unsigned long
-FrameJob::getXID (css::uno::Reference < css::frame::XFrame > xFrame)
-{
-	Reference< XSystemDependentWindowPeer > xWin( xFrame->getContainerWindow(), UNO_QUERY);
-
-	if (!xWin.is())
-		return 0;
-
-	sal_Int8 processID[16];
-	rtl_getGlobalProcessId( (sal_uInt8*)processID );
-	Sequence <signed char> pidSeq (processID, 16);
-
-	SystemDependentXWindow xWindow;
-	xWin->getWindowHandle (pidSeq, SYSTEM_XWINDOW) >>= xWindow;
-	
-	return xWindow.WindowHandle;
-}
-
-DbusmenuMenuitem*
-FrameJob::getRootMenuitem (Reference < XMenu > xMenu, gpointer helper)
-{
-	
-	DbusmenuMenuitem *root = dbusmenu_menuitem_new_with_id (0);
-	((FrameHelper*)helper)->setRootItem(root);
-	((FrameHelper*)helper)->rebuildMenu (xMenu, root);
-	
-	return root;
-}
